%% Input processing - initialization of vectors

n_materials = size(materials,2);    % number of materials
n_particles = 0;        % number of particles
V = 0;                  % volume of consituents
V_voids = 0;            % volume of voids
r_coated_temp = [];     % auxiliary vector for calculation of radii
r_coating_temp = [];    % auxiliary vector for calculation of radii
c_coated_temp = [];     % auxiliary vector for calculation of volume fractions
c_coating_temp = [];    % auxiliary vector for calculation of volume fractions
r_coated = [];          % radius of coated particles
r_coating = [];         % radius of coating of particles
p_i = [];           % percentage of inclusion volume consumed by coating
p_m = [];           % percentage of matrix volume consumed by coating
p_v = [];           % percentage of voids' volume consumed by coating
c_uncoated = [];    % volume fraction of uncoated particles
c_coated = [];      % volume fraction of coated particles
c_coating = [];      % volume fraction of coating
E_uncoated = [];    % Young's modulus of uncoated particles
E_coated = [];      % Young's modulus of coated particles
E_coating = [];     % Young's modulus of coating
E_voids = 0;       % Young's modulus of voids
nu_uncoated = [];   % Poisson's ratio of uncoated particles
nu_coated = [];     % Poisson's ratio of coated particles
nu_coating = [];    % Poisson's ratio of coating
nu_voids = 0;      % Poisson's ratio of voids

%% Input processing - collection of data
for i = 1:n_materials
    if strcmp(materials(i).code,'voids')
        V_voids = materials(i).volume/100;
    end
end

%for i = 1:n_materials
%    if strcmp(materials(i).code,'matrix')
%        materials(i).rho = materials(i).rho*(1-V_voids);
%    end
%end

for i = 1:n_materials
    if strcmp(materials(i).code,'matrix')
        n_particles = n_particles+1;
        V = V + materials(i).m/materials(i).rho;
        
    elseif strcmp(materials(i).code,'uncoated')
        n_particles = n_particles+1;
        V = V + materials(i).m/materials(i).rho;
        
    elseif strcmp(materials(i).code,'voids')
        n_particles = n_particles+1;
        V_voids = materials(i).volume/100;
        
    elseif strcmp(materials(i).code,'coated')
        n_particles = n_particles+size(materials(i).GC.frac,1)*materials(i).n_in_fraction*2;
        V = V + materials(i).m/materials(i).rho;  
    end
end

for i = 1:n_materials
    if strcmp(materials(i).code,'matrix')
        c_matrix = (materials(i).m/materials(i).rho)*(1-V_voids)/V;
        E_matrix = materials(i).E;
        nu_matrix = materials(i).nu;
        
    elseif strcmp(materials(i).code,'uncoated')
        c_uncoated = [c_uncoated; (materials(i).m/materials(i).rho)*(1-V_voids)/V];
        E_uncoated = [E_uncoated; materials(i).E];
        nu_uncoated = [nu_uncoated; materials(i).nu];
        
        % for crack generator
        fprintf('Mix is composed of matrix and %s grains.\n', materials(i).name)
        cAgg = sum(c_uncoated);
        GC_frac = materials(i).GC.frac;
        GC_amount = materials(i).GC.amount;
        
    elseif strcmp(materials(i).code,'voids')
        E_voids = materials(i).E;
        nu_voids = materials(i).nu;
        
    elseif strcmp(materials(i).code,'coated')
        c_coated_temp_local = (materials(i).m/materials(i).rho)*(1-V_voids)/V;
        V_local = sum(materials(i).GC.amount);
        c_local = materials(i).GC.amount/V_local*c_coated_temp_local;
        c_coated_temp = zeros(size(materials(i).GC.amount,1)*materials(i).n_in_fraction,1);
        E_temp = materials(i).E;
        nu_temp = materials(i).nu;
        
        for j = 1:size(materials(i).GC.amount,1)
            interval_size = materials(i).GC.frac(j,2)-materials(i).GC.frac(j,1);
            r_temp = zeros(materials(i).n_in_fraction,1);
            step = interval_size/materials(i).n_in_fraction;
            for k = 1:materials(i).n_in_fraction
                r_temp(k) = materials(i).GC.frac(j,1)-0.5*step+k*step;
                pos = (j-1)*materials(i).n_in_fraction+k;
                %if pos == 1
                %    c_coated_temp = c_local;
                %else
                    c_coated_temp(pos) = c_local(j)/materials(i).n_in_fraction;
                %end
            end
            r_coated_temp = [r_coated_temp; r_temp];
            E_coated_temp = ones(size(r_coated_temp,1),1)*materials(i).E; 
            nu_coated_temp = ones(size(r_coated_temp,1),1)*materials(i).nu; 
        end    
        c_coated = [c_coated; c_coated_temp];
        
        % for crack generator
        fprintf('Mix is composed of matrix and %s grains.\n', materials(i).name)
        cAgg = sum(c_coated);
        GC_frac = materials(i).GC.frac;
        GC_amount = materials(i).GC.amount;
        
        for l = 1:n_materials
            if strcmp(materials(l).code,'coating')
                if materials(l).ID == materials(i).ID
                    p_i_temp = materials(l).p_i*ones(size(r_coated_temp,1),1);
                    p_m_temp = materials(l).p_m*ones(size(r_coated_temp,1),1);
                    p_v_temp = materials(l).p_v*ones(size(r_coated_temp,1),1);
                    r_coating_temp = materials(l).thickness*ones(size(r_coated_temp,1),1);
                    E_coating_temp = ones(size(E_coated_temp,1),1)*materials(l).E;
                    nu_coating_temp = ones(size(nu_coated_temp,1),1)*materials(l).nu;
                    for m = 1:size(r_coating_temp,1)
                        r_coating_temp(m) = r_coating_temp(m)+r_coated_temp(m);
                    end
                    c_coating_temp = (ones(size(r_coated_temp,1),1)-(r_coated_temp.^3./r_coating_temp.^3)).*c_coated_temp;
                end
            end
        end
        r_coated = [r_coated; r_coated_temp];
        r_coating = [r_coating; r_coating_temp];
        r_coated_temp = [];
        r_coating_temp = [];
        E_coated = [E_coated; E_coated_temp];
        E_coating = [E_coating; E_coating_temp];
        nu_coated = [nu_coated; nu_coated_temp];
        nu_coating = [nu_coating; nu_coating_temp];
        c_coating = [c_coating; c_coating_temp];
        p_i = [p_i; p_i_temp*0.01];
        p_m = [p_m; p_m_temp*0.01];
        p_v = [p_v; p_v_temp*0.01];   
    end        
end
E_uncoated = [E_uncoated; E_voids];
nu_uncoated = [nu_uncoated; nu_voids];
c_uncoated = [c_uncoated; V_voids];

c_matrix_reduction = sum(c_coating.*p_m);
c_coated_reduction = c_coating.*p_i;
c_voids_reduction = sum(c_coating.*p_v);

c_matrix = c_matrix-c_matrix_reduction;
c_coated = c_coated-c_coated_reduction;
if size(c_uncoated,1) > 0
    c_uncoated(end) = c_uncoated(end)-c_voids_reduction;
end

%test = sum(c_uncoated)+sum(c_coated)+c_matrix+sum(c_coating)

%% Bulk and shear moduli

K_matrix = E_matrix/(3*(1-2*nu_matrix));
G_matrix = E_matrix/(2*(1+nu_matrix));

K_uncoated = E_uncoated./(3*(1-2*nu_uncoated));
G_uncoated = E_uncoated./(2*(1+nu_uncoated));

K_coated = E_coated./(3*(1-2*nu_coated));
G_coated = E_coated./(2*(1+nu_coated));

K_coating = E_coating./(3*(1-2*nu_coating));
G_coating = E_coating./(2*(1+nu_coating));