%% MT Homogenization 4 (accounting for cracks) (1/6/2013)
% created by Vaclav Nezerka

clear all
fprintf('\n\n')
addpath(fullfile(pwd,'files'))
addpath(fullfile(pwd,'functions'))
addpath(fullfile(pwd,['functions' filesep 'stiffness constants']))
addpath(fullfile(pwd,'input'))
addpath(fullfile(pwd,'CrackCounter3D'))
addpath(fullfile(pwd,['CrackCounter3D' filesep 'output']))
addpath(fullfile(pwd,['CrackCounter3D' filesep 'functions']))
addpath(fullfile(pwd,['CrackCounter3D' filesep 'functions' filesep 'MEX']))

%% Calculation

% input materials
run mortarSand_test

% sample dimensions
cubeSizeRVE = 30e-3;       % size of cubical RVE
minDistBetweenGrains = 0;  % minimum distance between grains
periodicBC = true;         % true = grains can intersect the RVE boundary

% utilization of C++ (MEX) functions (faster, no plotting !!! , only periodic BC available)
C_functions = true; % true = utilizes MEX function written in C++ for placement of grains
                    % false = utilizes only matlab (.m) functions

% setting the cracking criteria
criticalRatio = 0.7; % critical ratio of gap between aggregates to their mean diameter

% microstructure plotting options
figureOut = false; % true = plots the generated microstructure, false = no plotting
saveFigure = false; % true = saves the figure
uncrackedComparison = true; % true = compares with uncracked sample, false = no comparison

% read input data, simulate microstructure and calculate crack density
fprintf('MATERIAL: %s \n', input_name)
run input_processing
run crackCounter3D

% Homogenization of cracks
[E_matrix nu_matrix] = f_E_and_nu_from_K_and_G(K_matrix,G_matrix);
H_1111 = (crackDensity/E_matrix)*(16*(1-nu_matrix^2)*(10-3*nu_matrix)/(45*(2-nu_matrix))); % normal compliance due to presence of cracks
H_1212 = (crackDensity/E_matrix)*(32*(1-nu_matrix^2)*(5-nu_matrix)/(45*(2-nu_matrix))); % shear compliance due to presence of cracks
E_matrixCracks = (1/E_matrix+H_1111)^-1;
G_matrixCracks = (1/G_matrix+H_1212)^-1;
K_matrixCracks = E_matrixCracks*G_matrixCracks/(3*(3*G_matrixCracks-E_matrixCracks));

fprintf('The Young''s modulus of matrix was reduced by cracks from %.2f to %.2f MPa\n', E_matrix/1e6, E_matrixCracks/1e6)

moduliComparison = [K_matrix, G_matrix; K_matrixCracks, G_matrixCracks];
moduliDescription = {'uncracked matrix','cracked matrix'};

if uncrackedComparison
    [K_eff G_eff] = f_MT_incl_coated(...
        c_matrix, c_uncoated, c_coated, c_coating,...
        K_matrix, K_uncoated, K_coated, K_coating,...
        G_matrix, G_uncoated, G_coated, G_coating,...
        r_coated, r_coating);
    
    [E_eff nu_eff] = f_E_and_nu_from_K_and_G(K_eff,G_eff);
    
    fprintf('-- %s configuration --\n', char(moduliDescription(1)));
    fprintf('E_eff = %.1f MPa, ', E_eff/1e6)
    fprintf('nu_eff = %.3f\n', nu_eff)
    
    % cracked matrix
    [K_eff_cracks G_eff_cracks] = f_MT_incl_coated(...
        c_matrix, c_uncoated, c_coated, c_coating,...
        K_matrixCracks, K_uncoated, K_coated, K_coating,...
        G_matrixCracks, G_uncoated, G_coated, G_coating,...
        r_coated, r_coating);
    
    [E_eff_cracks nu_eff_cracks] = f_E_and_nu_from_K_and_G(K_eff_cracks,G_eff_cracks);
    
    fprintf('-- %s configuration --\n', char(moduliDescription(2)));
    fprintf('E_eff = %.1f MPa, ', E_eff_cracks/1e6)
    fprintf('nu_eff = %.3f\n', nu_eff_cracks)
else
    [K_eff_cracks G_eff_cracks] = f_MT_incl_coated(...
        c_matrix, c_uncoated, c_coated, c_coating,...
        K_matrixCracks, K_uncoated, K_coated, K_coating,...
        G_matrixCracks, G_uncoated, G_coated, G_coating,...
        r_coated, r_coating);
    
    [E_eff_cracks nu_eff_cracks] = f_E_and_nu_from_K_and_G(K_eff_cracks,G_eff_cracks);
    
    fprintf('-- %s configuration --\n', char(moduliDescription(2)));
    fprintf('E_eff = %.1f MPa, ', E_eff_cracks/1e6)
    fprintf('nu_eff = %.3f\n', nu_eff_cracks)
    fprintf('\n\n')
end